#ifndef Knopf_H
#define Knopf_H

#include "TextFeld.h"

namespace Framework
{
    class TextFeld;      //! TextFeld.h
    class AlphaFeld;     //! AlphaFeld.h
    class Text;          //! Text.h
    class Schrift;       //! Schrift.h
    class Rahmen;        //! Rahmen.h
    class LTDBDatei;     //! Dateisystem.h
    class Knopf;         //! aus dieser Datei
    class KontrollKnopf; //! aus dieser Datei
    class TextRenderer;

    //! Eine Zeichnung des 2D GUI Frameworks, die einen Knopf zeichnet, den der
    //! Benutzer drcken kann
    class Knopf : public TextFeld
    {
    public:
        class Style : public ZeichnungHintergrund::Style
        {
        public:
            //! Legt fest, ob der Text des Knopfes mehrere
            //! Farben haben darf
            static const __int64 MehrfarbigText = 0x0010000;
            //! Legt fest, ob der Knopf whrend er gedrckt
            //! wird eine Hintergrund Farbe hat
            static const __int64 KlickFarbe = 0x0020000;
            //! Legt fest, ob der Knopf whrend er gedrckt
            //! wird ein Bild als Hintergrund hat
            static const __int64 KlickBild = 0x0040000;
            //! Legt fest, ob der Knopf whrend er gedrckt
            //! wird fr das Zeichnen des Hintergrundes alpha
            //! blending verwendet
            static const __int64 KlickAlpha = 0x0080000;
            //! Legt fest, ob der Knopf whrend er gedrckt
            //! wird einen Farbbergang besitzt
            static const __int64 KlickBuffer = 0x0100000;

            //! Vereint di Flags Sichtbar, Erlaubt, Rahmen,
            //! Buffered, MehrfarbigText, KlickBuffer
            static const __int64 Normal = Sichtbar | Erlaubt | Rahmen | Buffered
                                        | MehrfarbigText | KlickBuffer;
        };

    private:
        int klickFarbe;
        Bild* klickBild;
        AlphaFeld* klickBuffer;
        int klickIndex;
        //! Verarbeitet Maus Nachrichten
        //! \param me Das Ereignis, was durch die Mauseingabe ausgelt wurde
        DLLEXPORT void doMausEreignis(MausEreignis& me, bool userRet) override;

    public:
        //! Konstruktor
        DLLEXPORT Knopf();
        //! Destruktor
        DLLEXPORT virtual ~Knopf();
        //! Setzt die Farbe, die als Hintergrund verwendet wird, wenn der Knopf
        //! gedrckt wird \param fc Die Farbe im A8R8G8B8 Format
        DLLEXPORT void setKlickFarbe(int fc);
        //! Setzt das Hintergrund Bild durch kopieren, das verwendet wird, wenn
        //! der Knopf gedrckt wird \param bild Das Bild, das kopiert werden
        //! soll
        DLLEXPORT void setKlickBild(Bild* bild);
        //! Setzt einen Zeiger auf das Hintergrund Bild, das verwendet wird,
        //! wenn der Knopf gedrckt wird \param bild Das neue Bild
        DLLEXPORT void setKlickBildZ(Bild* bild);
        //! Setzt einen Zeiger auf den Farbbergang, der verwendet wird, wenn
        //! der Knopf gedrckt wird \param af Der neue Farbbergang
        DLLEXPORT void setKBZ(AlphaFeld* af);
        //! Setzt die Strke des Farbbergangs, der verwendet wird, wenn der
        //! Knopf gedrckt wird \param st Die Strke
        DLLEXPORT void setKBStrength(int st);
        //! Setzt die Farbe des Farbbergangs, der verwendet wird, wenn der
        //! Knopf gedrckt wird \param fc Die Farbe im A8R8G8B8 Format
        DLLEXPORT void setKBFarbe(int fc);
        //! Verarbeitet Maus Nachrichten
        //! \param te Das Ereignis, was durch die Tastatur Eingabe ausgelt
        //! wurde
        DLLEXPORT void doTastaturEreignis(TastaturEreignis& te) override;
        //! Zeichnet das Objekt nach zRObj, falls es sichtbar ist
        //! \param zRObj Das Bild, in welches gezeichnet werden soll
        DLLEXPORT void render(Bild& zRObj) override;
        //! Gibt die Hintergrund Farbe im A8R8G8B8 Format zurck, die verwendet
        //! wird, wenn der Knopf gedrckt wird
        DLLEXPORT int getKlickFarbe() const;
        //! Gibt das Hintergrund Bild zurck, das verwendet wird, wenn der Knopf
        //! gedrckt wird
        DLLEXPORT Bild* getKlickBild() const;
        //! Gibt das Hintergrund Bild ohne erhhten Reference Counter zurck,
        //! das verwendet wird, wenn der Knopf gedrckt wird
        DLLEXPORT Bild* zKlickBild() const;
        //! Gibt den Farbbergang zurck, der verwendet wird, wenn der Knopf
        //! gedrckt wird
        DLLEXPORT AlphaFeld* getKB() const;
        //! Gibt den Farbbergang ohne erhhten Reference Counter zurck, der
        //! verwendet wird, wenn der Knopf gedrckt wird
        DLLEXPORT AlphaFeld* zKB() const;
        //! Gibt die Farbe des Farbbergangs im A8R8G8B8 Format zurck, der
        //! verwendet wird, wenn der Knopf gedrckt wird
        DLLEXPORT int getKBFarbe() const;
        //! Gibt die Strke des Farbbergangs zurck, der verwendet wird, wenn
        //! der Knopf gedrckt wird
        DLLEXPORT int getKBStrength() const;
        //! Kopiert den Knopf, so das er ohne auswirkungen auf das Original
        //! verndert werden kann
        DLLEXPORT Zeichnung* dublizieren() const override;
    };

    //! Eine Zeichnung des 2D GUI Frameworks, die einen Auswhlbares Kstchen
    //! mit beschriftung zeichnet
    class KontrollKnopf : public ZeichnungHintergrund
    {
    public:
        class Style : public ZeichnungHintergrund::Style
        {
        public:
            //! Legt fest, ob das Kstchen ausgewhlt ist
            static const __int64 Selected = 0x1000000;
            //! Legt Fest, ob der Beschriftungs Text mehrfarbig
            //! sein kann
            static const __int64 MehrfarbigText = 0x001000;
            //! Legt fest, ob das Kstchen whrend ed gedrckt
            //! wird eine Hintergrund Farbe hat
            static const __int64 KlickFarbe = 0x002000;
            //! Legt fest, ob das Kstchen beim Klicken ein
            //! Hintergrund Bild hat
            static const __int64 KlickBild = 0x004000;
            //! Legt fest, ob fr den Hintergrund des Kstchens
            //! beim Klicken alpha blending verwendet wird
            static const __int64 KlickAlpha = 0x008000;
            //! Legt fest, ob das Kstchen beim Klicken einen
            //! Farbbergang hat
            static const __int64 KlickBuffer = 0x010000;
            //! Legt fest, ob das Kstchen wrhend es ausgewhlt
            //! ist eine Hintergrund Farbe hat
            static const __int64 SelectFarbe = 0x020000;
            //! Legt fest, ob das Kstchen wrhend es ausgewhlt
            //! ist ein Hintergrund Bild hat
            static const __int64 SelectBild = 0x040000;
            //! Legt fest, ob das Kstchen wrhend es ausgewhlt
            //! ist fr den Hintergrund alpha blending verwendet
            static const __int64 SelectAlpha = 0x080000;
            //! Legt fest, ob das Kstchen wrhend es ausgewhlt
            //! ist einen Farbbergang hat
            static const __int64 SelectBuffer = 0x100000;
            //! Legt fest, ob das Kstchen wrhend es ausgewhlt
            //! ist eine andere Beschriftung hat
            static const __int64 SelectText = 0x200000;
            //! Legt fest, ob das Kstchen eine Mehrzeilige
            //! Beschriftung haben kann
            static const __int64 MehrzeiligText = 0x400000;
            //! Legt fest, ob gerade mit der Maus auf das
            //! Kstchen geklickt wird. (Wird vom Kstchen
            //! selbst verwaltet)
            static const __int64 MausKlick = 0x800000;

            //! Vereint die Flags Sichtbar, Erlaubt, Rahmen,
            //! KlickBuffer
            static const __int64 Normal
                = Sichtbar | Erlaubt | Rahmen | KlickBuffer;
        };

    private:
        Text* txt;
        Text* sTxt;
        int sBgF;
        int kBgF;
        Bild* sBgB;
        Bild* kBgB;
        AlphaFeld* sAf;
        AlphaFeld* kAf;
        Bild* kasten;
        Bild* sKasten;
        TextRenderer* textRd;
        int sF;
        int sGr;
        //! Verarbeitet Maus Nachrichten
        //! \param me Das Ereignis, was durch die Mauseingabe ausgelt wurde
        DLLEXPORT void doMausEreignis(MausEreignis& me, bool userRet) override;

    public:
        //! Konstruktor
        DLLEXPORT KontrollKnopf();
        //! Destruktor
        DLLEXPORT virtual ~KontrollKnopf();
        //! Setzt den Zeiger auf den Beschriftungs Text
        //! \param txt Der Text
        DLLEXPORT void setTextZ(Text* txt);
        //! Setzt den Beschriftungs Text
        //! \param txt Der Text
        DLLEXPORT void setText(Text* txt);
        //! Setzt den Beschriftungs Text
        //! \param txt Der Text
        DLLEXPORT void setText(const char* txt);
        //! Setzt den Zeiger auf den Beschriftungs Text, der angezeigt wird,
        //! whrend das Kstchen ausgewhlt ist \param txt Der Text
        DLLEXPORT void setSTextZ(Text* txt);
        //! Setzt den Beschriftungs Text, der angezeigt wird, whrend das
        //! Kstchen ausgewhlt ist \param txt Der Text
        DLLEXPORT void setSText(Text* txt);
        //! Setzt den Beschriftungs Text, der angezeigt wird, whrend das
        //! Kstchen ausgewhlt ist \param txt Der Text
        DLLEXPORT void setSText(const char* txt);
        //! Setzt den verwendeten TextRenderer
        //! \param textRd Der Textrenderer
        DLLEXPORT void setTextRendererZ(TextRenderer* textRd);
        //! Setzt die Schrift, die fr die Beschriftung verwendet werden soll
        //! \param schrift Die Schrift
        DLLEXPORT void setSchriftZ(Schrift* schrift);
        //! Setzt die Farbe der Schrift, die fr die Beschriftung verwendet
        //! werden soll \param f Die Farbe im A8R8G8B8 Format
        DLLEXPORT void setSFarbe(int f);
        //! Setzt die Gre der Schrift, die fr die Beschriftung verwendet
        //! werden soll \param f Die Hhe einer Zeile in Pixeln
        DLLEXPORT void setSSize(int gr);
        //! Setzt die Hintergrund Farbe, die verwendet wird, whrend das
        //! Kstchen ausgewhlt ist \param f Die Farbe im A8R8G8B8 Format
        DLLEXPORT void setSBgFarbe(int f);
        //! Setzt die Hintergrund Farbe, die verwendet wird, whrend auf das
        //! Kstchen geklickt wird \param f Die Farbe im A8R8G8B8 Format
        DLLEXPORT void setKBgFarbe(int f);
        //! Setzt einen Zeiger auf das Hintergrund Bild, das verwendet wird,
        //! whrend das Kstchen ausgewhlt ist \param b Das Bild
        DLLEXPORT void setSBgBildZ(Bild* b);
        //! Setzt das Hintergrund Bild durch kopieren, das verwendet wird,
        //! whrend das Kstchen ausgewhlt ist \param b Das Bild, das kopiert
        //! werden soll
        DLLEXPORT void setSBgBild(Bild* b);
        //! Setzt einen Zeiger auf das Hintergrund Bild, das verwendet wird,
        //! whrend auf das Kstchen geklickt wird \param b Das Bild
        DLLEXPORT void setKBgBildZ(Bild* b);
        //! Setzt das Hintergrund Bild durch kopieren, das verwendet wird,
        //! whrend auf das Kstchen geklickt wird \param b Das Bild, das
        //! kopiert werden soll
        DLLEXPORT void setKBgBild(Bild* b);
        //! Setzt einen Zeiger auf den Farbbergang, der verwendet wird, whrend
        //! das Kstchen ausgewhlt ist \param af Der Farbbergang
        DLLEXPORT void setSAlphaFeldZ(AlphaFeld* af);
        //! Setzt die Farbe des Farbbergangs, der verwendet wird, whrend das
        //! Kstchen ausgewhlt ist \param f Die Farbe im A8R8G8B8 Format
        DLLEXPORT void setSAFFarbe(int f);
        //! Setzt die Strke des Farbbergangs, der verwendet wird, whrend das
        //! Kstchen ausgewhlt ist \param st Die Strke
        DLLEXPORT void setSAFStrength(int st);
        //! Setzt einen Zeiger auf den Farbbergang, der verwendet wird, whrend
        //! auf das Kstchen geklickt wird \param af Der Farbbergang
        DLLEXPORT void setKAlphaFeldZ(AlphaFeld* af);
        //! Setzt die Farbe des Farbbergangs, der verwendet wird, whrend auf
        //! das Kstchen geklickt wird \param f Die Farbe im A8R8G8B8 Format
        DLLEXPORT void setKAFFarbe(int f);
        //! Setzt die Strke des Farbbergangs, der verwendet wird, whrend auf
        //! das Kstchen geklickt wird \param st Die Strke
        DLLEXPORT void setKAFStrength(int st);
        //! Ldt die Bilder "kasten.gif" und "skasten.gif" aus einer LTDB Datei
        //! Das Bild kasten.gif wird gezeichnet, wenn das Kstchen nicht
        //! ausgewhlt ist. Das Bild skasten.gif wird gezeichnet, wenn das
        //! Kstchen ausgewhlt wird \param zDat Die LTDB Datei
        DLLEXPORT void loadData(LTDBDatei* zDat);
        //! Ldt die Bilder "kasten.gif" und "skasten.gif" aus einer LTDB Datei
        //! Das Bild kasten.gif wird gezeichnet, wenn das Kstchen nicht
        //! ausgewhlt ist. Das Bild skasten.gif wird gezeichnet, wenn das
        //! Kstchen ausgewhlt wird \param ltdb Der Pfad zur LTDB Datei
        DLLEXPORT void loadData(const char* ltdb);
        //! Zeichnet das Objekt nach zRObj, falls es sichtbar ist
        //! \param zRObj Das Bild, in welches gezeichnet werden soll
        DLLEXPORT void render(Bild& zRObj) override;
        //! Gibt die beschriftung zurck
        DLLEXPORT Text* getText() const;
        //! Gibt die beschriftung ohne erhhten Reference Counter zurck
        DLLEXPORT Text* zText() const;
        //! Gibt die beschriftung zurck, die verwendet wird, whrend das
        //! Kstchen ausgewhlt ist
        DLLEXPORT Text* getSText() const;
        //! Gibt die beschriftung ohne erhhten Reference Counter zurck, die
        //! verwendet wird, whrend das Kstchen ausgewhlt ist
        DLLEXPORT Text* zSText() const;
        //! Gibt die verwendete Schrift zurck
        DLLEXPORT Schrift* getSchrift() const;
        //! Gibt die verwendete Schrift ohne Reference Counter zurck
        DLLEXPORT Schrift* zSchrift() const;
        //! Gibt die Schrift Farbe im A8R8G8B8 Format zurck
        DLLEXPORT int getSFarbe() const;
        //! Gibt die Schrift Gre zurck
        DLLEXPORT int getSSize() const;
        //! Gibt die Hintergrund Farbe im A8R8G8B8 Format zurck, die verwendet
        //! wird, whrend das Kstchen ausgewhlt ist
        DLLEXPORT int getSBgFarbe() const;
        //! Gibt die Hintergrund Farbe im A8R8G8B8 Format zurck, die verwendet
        //! wird, whrend auf das Kstchen geklickt wird
        DLLEXPORT int getKBgFarbe() const;
        //! Gibt das Hintergrund Bild zurck, das verwendet wird, whrend das
        //! Kstchen ausgewhlt ist
        DLLEXPORT Bild* getSBgBild() const;
        //! Gibt das Hintergrund Bild ohne erhhten Reference Counter zurck,
        //! das verwendet wird, whrend das Kstchen ausgewhlt ist
        DLLEXPORT Bild* zSBgBild() const;
        //! Gibt das Hintergrund Bild zurck, das verwendet wird, whrend auf
        //! das Kstchen geklickt wird
        DLLEXPORT Bild* getKBgBild() const;
        //! Gibt das Hintergrund Bild ohne erhhten Reference Counter zurck,
        //! das verwendet wird, whrend auf das Kstchen geklickt wird
        DLLEXPORT Bild* zKBgBild() const;
        //! Gibt den Farbbergang zurck, der verwendet wird, whrend das
        //! Kstchen ausgewhlt ist
        DLLEXPORT AlphaFeld* getSAlphaFeld() const;
        //! Gibt den Farbbergang ohne erhhten Reference COunter zurck, der
        //! verwendet wird, whrend das Kstchen ausgewhlt ist
        DLLEXPORT AlphaFeld* zSAlphaFeld() const;
        //! Gibt den Farbbergang zurck, der verwendet wird, whrend auf das
        //! Kstchen geklickt wird
        DLLEXPORT AlphaFeld* getKAlphaFeld() const;
        //! Gibt den Farbbergang ohne erhhten Reference COunter zurck, der
        //! verwendet wird, whrend auf das Kstchen geklickt wird
        DLLEXPORT AlphaFeld* zKAlphaFeld() const;
    };
} // namespace Framework

#endif